/**
 * debounce
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-debounce.html&module=%2Ffunctions-debounce.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = debounce(res => console.log(res), delay);

  listener('1');
  listener('2');
  listener('3');
  listener(`4 ${new Date()}`);

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
const debounce = (fn, /* istanbul ignore next */ delay = 0) => {
    let timer;
    return (...args) => {
        window.clearTimeout(timer);
        timer = window.setTimeout(() => {
            fn.apply(null, args);
        }, delay);
    };
};

/**
 * deepFreeze
 * @param {any} target
 * @memberof module:functions
 * @function
 */
const deepFreeze = (target) => {
    if (Object.isFrozen(target)) {
        return target;
    }
    if (!Array.isArray(target) && Object.getPrototypeOf(target) !== Object.getPrototypeOf({})) {
        if (Object.getPrototypeOf(target) !== Object.getPrototypeOf(Function)) {
            throw new Error('not allow types');
        }
    }
    Object.freeze(target);
    Object.keys(target).forEach(key => deepFreeze(target[key]));
    return target;
};

/**
 * empty function
 * @memberof module:functions
 * @alias noop
 */
function noop() { }

/**
 * throttle
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-throttle.html&module=%2Ffunctions-throttle.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = throttle(res => console.log(res), delay);

  listener(`1 ${new Date()}`);
  listener('2');
  listener('3');
  listener('4');

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
const throttle = (fn, /* istanbul ignore next */ delay = 0) => {
    const caller = (args) => {
        fn.apply(null, args);
        base = +new Date();
    };
    let base = null;
    let isLeading = true;
    let stamp = null;
    return (...args) => {
        if (isLeading) {
            caller(args);
            isLeading = false;
            return;
        }
        stamp = +new Date();
        /* istanbul ignore next */
        base = base || stamp;
        if ((stamp - base) >= delay) {
            /* istanbul ignore next */
            caller(args);
        }
    };
};

/**
 * @module module:functions
 */

/**
 * EventEmitter 의 기능과 동일한 맥락으로, 간단히 구성
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fevent-EventEmitter.html&module=%2Fevent-EventEmitter.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class EventEmitter
 * @alias event/EventEmitter
 * @memberof event
 * @see https://www.npmjs.com/package/events
 * @example
import { EventEmitter } from '@nonoll/code-snippet/event';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let ee;
const forExample = () => {
  const doc = window.document;

  const onButton = createElement({ tag: 'button', text: 'on button' });
  const offButton = createElement({ tag: 'button', text: 'off button' });
  const emitButton = createElement({ tag: 'button', text: 'emit button' });
  const EVENT_NAME = 'EVENT_EXAMPLE';

  doc.body.appendChild(onButton);
  doc.body.appendChild(offButton);
  doc.body.appendChild(emitButton);

  ee = new EventEmitter();
  ee.on(EVENT_NAME, res => console.log(res));

  onButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('onButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
    ee.on(EVENT_NAME, res => console.log(res));
  });

  offButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('offButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
  });

  emitButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('emitButton clicked');
    if (!ee) {
      return;
    }
    ee.emit(EVENT_NAME, new Date());
  });
}

forExample();
 */
class EventEmitter {
    constructor() {
        this.events = [];
    }
    /**
     * 이벤트 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    on(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        this.events.push({ eventName, listener, context });
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    off(eventName, listener) {
        // tslint:disable-next-line: max-line-length
        const matched = this.events.findIndex(regEvent => regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true));
        // tslint:disable-next-line: no-bitwise
        if (!~matched) {
            return false;
        }
        this.events.splice(matched, 1);
        return this;
    }
    /**
     * 이벤트 1회 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    once(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        const onceWrapper = (...values) => {
            this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    emit(eventName, ...values) {
        this.events.forEach(regEvent => {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {any} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    fire(eventName, values) {
        this.emit(eventName, values);
        return this;
    }
}

/**
 * @alias event
 */

/**
 * @typedef {String} MutationRecordType
 * @alias MutationRecordType
 * @memberof MutationObserver
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @property {String} childList
 * @property {String} attributes
 * @property {String} characterData
 * @property {String} subtree
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MutationRecordType}
 */
var MutationRecordType;
(function (MutationRecordType) {
    MutationRecordType["childList"] = "childList";
    MutationRecordType["attributes"] = "attributes";
    MutationRecordType["characterData"] = "characterData";
    MutationRecordType["subtree"] = "subtree";
})(MutationRecordType || (MutationRecordType = {}));
/**
 * MutationEvent Types
 * @event MutationObserver#MUTATION_EVENTS
 * @memberof MutationObserver
 * @property {String} CHANGE_CHILD_LIST - childList 변경 시점
 * @property {String} CHANGE_SUBTREE - subtree 변경 시점
 * @property {String} CHANGE_ATTRIBUTES - attributes 변경 시점
 * @property {String} CHANGE_CHARACTER_DATA - data 변경 시점
 * @property {String} CHANGE - 기타 변경 시점
 * @property {String} WILD_CARD - 모든 변경 시점
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MUTATION_EVENTS}
 */
var MUTATION_EVENTS;
(function (MUTATION_EVENTS) {
    MUTATION_EVENTS["CHANGE_CHILD_LIST"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHILD_LIST";
    MUTATION_EVENTS["CHANGE_SUBTREE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_SUBTREE";
    MUTATION_EVENTS["CHANGE_ATTRIBUTES"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_ATTRIBUTES";
    MUTATION_EVENTS["CHANGE_CHARACTER_DATA"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHARACTER_DATA";
    MUTATION_EVENTS["CHANGE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE";
    MUTATION_EVENTS["WILD_CARD"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_WILCD_CARD";
})(MUTATION_EVENTS || (MUTATION_EVENTS = {}));
class PureMutationObserver {
    constructor(callback) {
        // TODO polyfill
        // const MutationObserver = require('mutation-observer');
        // this.pure = new MutationObserver(callback);
        this.pure = new window.MutationObserver(callback);
    }
    observe(target, options) {
        this.pure.observe(target, options);
    }
    disconnect() {
        this.pure.disconnect();
    }
}
/**
 * target 으로 설정된 element 의 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-MutationObserver.html&module=%2Fobserver-MutationObserver.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class MutationObserver
 * @alias observer/MutationObserver
 * @extends EventEmitter
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver
 * @see https://github.com/webmodules/mutation-observer
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @param {Partial<IMutationObserverExtOption>} option
 * @param {HTMLElement} [option.target=null] target
 * @param {Number} [option.debounce=300] debounce
 * @param {Function|null} [option.callback={@link noop}] callback function
 * @param {Object} [option.options={}]
 * @param {Boolean} [option.options.childList]
 * @param {Boolean} [option.options.attributes]
 * @param {Boolean} [option.options.characterData]
 * @param {Boolean} [option.options.subtree]
 * @param {Boolean} [option.options.attributeOldValue]
 * @param {Boolean} [option.options.characterDataOldValue]
 * @param {Array.<String>} [option.options.attributeFilter] [MDN attributeFilter]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationObserverInit/attributeFilter}
 * @example
import { MutationObserver, MUTATION_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

const forExample = () => {
  if (observer) {
    console.log('already example');
    return;
  }

  const doc = window.document;

  const target = createElement({ id: 'example_target', style: 'border: 1px solid red' });
  doc.body.appendChild(target);

  const options = {
    childList: true,
    subtree: true
  };

  observer = new MutationObserver({ target, options });

  observer.on(MUTATION_EVENTS.WILD_CARD, (type, values) => {
    console.log('wildCard', type, values);
  }).on(MUTATION_EVENTS.CHANGE_CHILD_LIST, values => {
    console.log('childList', values);
  });

  observer.attach();

  const attachButton = createElement({ tag: 'button', text: 'observer attach' });
  const detachButton = createElement({ tag: 'button', text: 'observer detach' });
  const appendButton = createElement({ tag: 'button', text: 'append' });

  doc.body.appendChild(attachButton);
  doc.body.appendChild(detachButton);
  doc.body.appendChild(appendButton);

  attachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('attachButton clicked');
    if (!observer) {
      return;
    }
    observer.attach();
  });

  detachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('detachButton clicked');
    if (!observer) {
      return;
    }
    observer.detach();
  });

  appendButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('appendButton clicked');
    if (!observer) {
      return;
    }
    const input = createElement({ tag: 'input', value: `${+new Date()}` });
    target.appendChild(input);
  });
}

forExample();
 */
class MutationObserver extends EventEmitter {
    constructor({ target = null, debounce: debounce$1 = 300, callback = noop, options = {} }) {
        super();
        const defaultOptions = { childList: true, subtree: true };
        this.target = target;
        this.options = Object.assign({}, defaultOptions, options);
        this.onMutationObserverListener = this.onMutationObserverListener.bind(this);
        /* istanbul ignore next: for noop */
        this.callback = callback || noop;
        if (debounce$1) {
            this.observer = new PureMutationObserver(debounce(this.onMutationObserverListener, debounce$1));
        }
        else {
            this.observer = new PureMutationObserver(this.onMutationObserverListener);
        }
    }
    /**
     * 이벤트 감지 설정
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.attach();
     */
    attach() {
        this.observer.observe(this.target, this.options);
        return this;
    }
    /**
     * 이벤트 감지 등록
     * @override
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @memberof MutationObserver
     * @listens MutationObserver#MUTATION_EVENTS
     * @returns {MutationObserver}
     */
    on(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        super.on(eventName, listener, context);
        return this;
    }
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @param {MutationRecord[]} mutations [MDN MutationRecord]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationRecord}
     * @memberof MutationObserver
     * @fires MutationObserver#MUTATION_EVENTS
     */
    onMutationObserverListener(mutations) {
        mutations.forEach(mutation => {
            const type = mutation.type;
            const values = { type, mutation };
            switch (type) {
                case MutationRecordType.childList:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    break;
                case MutationRecordType.subtree:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    break;
                case MutationRecordType.attributes:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    break;
                case MutationRecordType.characterData:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    break;
                /* istanbul ignore next: for default */
                default:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE, type, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
            }
        });
    }
    /**
     * 이벤트 감지 해제
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.detach();
     */
    detach() {
        if (this.observer) {
            this.observer.disconnect();
        }
        return this;
    }
    /**
     * destory
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.destroy();
     */
    destroy() {
        this.detach();
        this.observer = null;
        return this;
    }
}

const doc = window.document;
/**
 * 실행된 browser 기준으로, visibilityChange event 에 맞는 vendorPrefix 반환
 * @memberof VisibilityChangeObserver
 * @export
 * @returns {String}
 */
function getVendorPrefix() {
    let prefix = '';
    if (typeof doc.hidden !== 'undefined') {
        prefix = '';
    } /* istanbul ignore next */
    else if (typeof doc.msHidden !== 'undefined') {
        prefix = 'ms';
    } /* istanbul ignore next */
    else if (typeof doc.webkitHidden !== 'undefined') {
        prefix = 'webkit';
    }
    return prefix;
}
/**
 * getVendorPrefix 의 값
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
const VENDOR_PREFIX = getVendorPrefix();
/**
 * browser 상의 visibilitychange event 명
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
const VISIBILITY_EVENT_NAME = `${VENDOR_PREFIX}visibilitychange`;
/**
 * browser 상의 visibility hidden 시점의 event 명
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
const HIDDEN_METHOD_NAME = VENDOR_PREFIX ? /* istanbul ignore next */ `${VENDOR_PREFIX}Hidden` : 'hidden';
/**
 * VisibilityChange Event Types
 * @event VisibilityChangeObserver#VISIBILITY_EVENTS
 * @memberof VisibilityChangeObserver
 * @property {String} CHANGE - 변경 시점
 * @property {String} GET_STATUS - 현재 상태값 체크
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {VISIBILITY_EVENTS}
 */
var VISIBILITY_EVENTS;
(function (VISIBILITY_EVENTS) {
    VISIBILITY_EVENTS["CHANGE"] = "VISIBILITY_CHANGE-EVENTS-CHANGE";
    VISIBILITY_EVENTS["GET_STATUS"] = "VISIBILITY_CHANGE-EVENTS-GET_STATUS";
})(VISIBILITY_EVENTS || (VISIBILITY_EVENTS = {}));
/**
 * browser 의 visibility 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-VisibilityChange.html&module=%2Fobserver-VisibilityChange.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class VisibilityChangeObserver
 * @alias observer/VisibilityChangeObserver
 * @extends {EventEmitter}
 * @see https://developer.mozilla.org/en-US/docs/Web/API/Page_Visibility_API
 * @see https://caniuse.com/#search=visibilityState
 * @throws {Error} browser 에서 지원하지 못하는 경우 - throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
 * @example
import { VisibilityChangeObserver, VISIBILITY_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

const forExample = () => {
  if (observer) {
    console.log('already example');
    return;
  }

  const doc = window.document;

  observer = new VisibilityChangeObserver();

  observer.on(VISIBILITY_EVENTS.CHANGE, ({ isHidden, isShow }) => {
    console.log('change', isHidden, isShow);
  });

  observer.attach();
  observer.emit(VISIBILITY_EVENTS.GET_STATUS);

  const statusButton = createElement({ tag: 'button', text: 'status' });
  const openButton = createElement({ tag: 'button', text: 'window open' });

  doc.body.appendChild(statusButton);
  doc.body.appendChild(openButton);

  statusButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('statusButton clicked');
    if (!observer) {
      return;
    }
    // observer.emit(VISIBILITY_EVENTS.GET_STATUS);
    const { isHidden, isShow } = observer.getStatus();
    console.log('getStatus', isHidden, isShow);
  });

  openButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('openButton clicked');
    if (!observer) {
      return;
    }
    const browser = window.open('https://nonoll.github.io/code-snippet/');
    if (!browser) {
      console.error('팝업이 차단되어 있습니다.');
    }
  });
}

forExample();
 */
class VisibilityChangeObserver extends EventEmitter {
    constructor() {
        super();
        /* istanbul ignore next: for not support error */
        if (!VisibilityChangeObserver.isSupportVisibilityChange()) {
            throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
        }
        this.onVisibilityChangeListener = this.onVisibilityChangeListener.bind(this);
    }
    /**
     * visibilitychange event 명 반환
     * @static
     * @returns {String}
     * @memberof VisibilityChangeObserver
     */
    static visibilityEventName() {
        return `${VENDOR_PREFIX}visibilitychange`;
    }
    /**
     * visibility hidden event 명 반환
     * @static
     * @returns {String}
     * @memberof VisibilityChangeObserver
     */
    static hiddenMethodName() {
        return HIDDEN_METHOD_NAME;
    }
    /**
     * visibility change 지원 여부 반환
     * @static
     * @returns {boolean}
     * @memberof VisibilityChangeObserver
     */
    static isSupportVisibilityChange() {
        return typeof doc[VisibilityChangeObserver.hiddenMethodName()] !== 'undefined';
    }
    /**
     * @private
     * @memberof VisibilityChangeObserver
     * @see IVisibilityStatus
     * @fires VisibilityChangeObserver#VISIBILITY_EVENTS
     */
    onVisibilityChangeListener() {
        const isHidden = this.isHidden();
        this.emit(VISIBILITY_EVENTS.CHANGE, { isHidden, isShow: !isHidden });
    }
    /**
     * @private
     * @returns {Boolean}
     * @memberof VisibilityChangeObserver
     */
    isHidden() {
        return doc[VisibilityChangeObserver.hiddenMethodName()];
    }
    /**
     * 현재 상태를 반환
     * @returns {IVisibilityStatus}
     * @memberof VisibilityChangeObserver
     */
    getStatus() {
        this.emit(VISIBILITY_EVENTS.GET_STATUS);
        const isHidden = this.isHidden();
        return { isHidden, isShow: !isHidden };
    }
    /**
     * 이벤트 감지 설정
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    attach() {
        doc.addEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.on(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    detach() {
        doc.removeEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.off(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    /**
     * destory
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    destroy() {
        this.detach();
        return this;
    }
}

/*
 * Public API Surface of @nonoll/code-snippet/observer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HIDDEN_METHOD_NAME, MUTATION_EVENTS, MutationObserver, MutationRecordType, VENDOR_PREFIX, VISIBILITY_EVENTS, VISIBILITY_EVENT_NAME, VisibilityChangeObserver, getVendorPrefix };
//# sourceMappingURL=nonoll-code-snippet-observer.js.map
